/*
 * Created on 20.10.2005 for PIROL
 *
 * SVN header information:
 *  $Author: LBST-PF-3\orahn $
 *  $Rev: 2453 $
 *  $Date: 2006-09-12 15:47:25 +0200 (Di, 12 Sep 2006) $
 *  $Id: StandardPirolToolBox.java 2453 2006-09-12 13:47:25Z LBST-PF-3\orahn $
 */
package de.fhOsnabrueck.jump.pirol.utilities.toolBoxes;

import java.awt.Image;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.image.BufferedImage;
import java.io.InputStream;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;

import com.vividsolutions.jump.workbench.plugin.PlugInContext;
import com.vividsolutions.jump.workbench.ui.GUIUtil;
import com.vividsolutions.jump.workbench.ui.LayerNamePanel;
import com.vividsolutions.jump.workbench.ui.LayerNamePanelListener;
import com.vividsolutions.jump.workbench.ui.toolbox.ToolboxDialog;
import com.vividsolutions.jump.workbench.ui.toolbox.ToolboxPlugIn;

import de.fhOsnabrueck.jump.pirol.utilities.apiTools.PlugInContextTools;
import de.fhOsnabrueck.jump.pirol.utilities.i18n.PirolPlugInMessages;
import de.fhOsnabrueck.jump.pirol.utilities.settings.PirolPlugInSettings;

/**
 * Abstract base class for ToolBoxes in the PIROL context.
 *
 * @author Ole Rahn
 * <br>
 * <br>FH Osnabr&uuml;ck - University of Applied Sciences Osnabr&uuml;ck,
 * <br>Project: PIROL (2005),
 * <br>Subproject: Daten- und Wissensmanagement
 * 
 * @version $Rev: 2453 $
 * 
 */
public abstract class StandardPirolToolBox extends ToolboxPlugIn implements LayerNamePanelListener, FocusListener {
    
    protected ImageIcon ICON = null;
    protected String imgName = "rasterToolBox.png";
    protected ToolboxDialog toolbox = null;
    
    protected String KEY = null;
    
    protected LayerNamePanel oldLayerNamePanel = null; 
    
    public StandardPirolToolBox(String imgName){
        this.KEY = this.getClass().getName();
        this.imgName = imgName;
        
    }
    
    /**
     * Name of the Tool to be shown in the menus or as a tooltip in JUMP.<br>
     * Looks for a key (the PlugIn's name with no path) in the i18n resources, if none is found
     * the standard jump name generation will be used.
     */
    public String getName() {
        try {
            return PirolPlugInMessages.getString(this.getShortClassName());
        } catch (RuntimeException e) {
            return super.getName();
        }
        
    }
    
    public String getShortClassName(){
        return this.getClass().getSimpleName();
    }
    
    public void initialize(PlugInContext context) throws Exception {
        if ( this.ICON == null ){
            InputStream in = getClass().getResourceAsStream(imgName);
            Image img = ImageIO.read(in).getScaledInstance(PirolPlugInSettings.StandardToolIconWidth, PirolPlugInSettings.StandardToolIconHeight, BufferedImage.SCALE_SMOOTH);
            this.ICON = new ImageIcon(img);
        }
        
        context.getWorkbenchContext().getWorkbench().getBlackboard().put(this.KEY, this);
        
        this.createMainMenuItem(new String[] { PirolPlugInSettings.getName_PirolMenu() },
                GUIUtil.toSmallIcon(this.ICON), context.getWorkbenchContext());

    }

    /**
     *@inheritDoc
     */
    protected abstract void initializeToolbox(ToolboxDialog toolbox);
    
    

    public void layerSelectionChanged() {
    	this.getToolbox(PlugInContextTools.getContext(toolbox).getWorkbenchContext()).updateEnabledState();
    }

    public void focusGained(FocusEvent e) {
    	if (oldLayerNamePanel!=null){
    		oldLayerNamePanel.removeListener(this);
    	}
        PlugInContextTools.getContext(toolbox).getLayerNamePanel().addListener(this);

        this.getToolbox(PlugInContextTools.getContext(toolbox).getWorkbenchContext()).updateEnabledState();
    }

    public void focusLost(FocusEvent e) {
    	oldLayerNamePanel = PlugInContextTools.getContext(toolbox).getLayerNamePanel();

    	this.getToolbox(PlugInContextTools.getContext(toolbox).getWorkbenchContext()).updateEnabledState();
    }

}
